// -*- Mode: C++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
/* connection.h: Qt wrapper for DBusConnection
 *
 * Copyright (C) 2003  Zack Rusin <zack@kde.org>
 *               2005  James Thorniley <james@uncommonlygood.co.uk>
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef DBUS_QT_CONNECTION_H
#define DBUS_QT_CONNECTION_H


#include <qobject.h>
#include <qstring.h>

#include "dbus/dbus.h"

namespace DBusQt {
  namespace Internal {
    class Integrator;
  }

  class Message;
  class Object;
  
  /**
  @brief Provides a connection to a D-BUS message bus.
  
  A connection must be acquired before an app can communicate
  over D-BUS. The easiest way to get a connection is to
  use the {@link Connection( DBusBusType, QObject* ) bus type based constructor}. 
  You can then use send and friends
  to send {@link Message messages} over D-BUS, or you can register
  a new {@link Object object} with registerObject.
  */
  class Connection : public QObject
  {
    Q_OBJECT
    
  public:
  
    Connection( QObject *parent =0 );
    Connection( const QString& host,
                QObject *parent = 0 );
                
    /**
    @brief Opens a connection to the main session or system message bus.
    
    @param type The message bus to connect to, either @arg @c DBUS_BUS_SESSION for the
    session bus; or @arg @c DBUS_BUS_SYSTEM for the system bus.
    @param parent The QObject parent for this QObject.
    */
    Connection( DBusBusType type, QObject* parent = 0 );
    ~Connection();
    
    bool isConnected() const;
    bool isAuthenticated() const;

    Message borrowMessage();
    Message popMessage();
    void stealBorrowMessage( const Message& );
    void dbus_connection_setup_with_qt_main (DBusConnection *connection);

    /**
    @brief Registers an object on the message bus.
    
    The object will be called when messages are sent to the path specified by
    @c path and @c name.
    
    @param object A pointer to the object to register. This pointer must not be
    deleted without first unregistering it from the bus with unregisterObject()
    @param path The path to the object, e.g. "/org/freedesktop/"
    @param name The name of the object, e.g. "DBus"
    @returns False if the object could not be registered.
    @todo You can only register one object on any particular path,
    is this the right behaviour?
    @todo Deal with badly formed names / paths*/
    bool registerObject( Object *object, const QString &path, const QString &name );
    
    /**
    @brief Unregisters an object so it will no longer recieve messages
    
    The object with @c name at the given @c path will no longer receive
    messages from this connection.
    
    @param path The path of the object to unregister
    @param name The name of the object to unregister
    @returns false if no object is registered at the given path / name
    */
    bool unregisterObject( const QString &path, const QString &name );
    
    /**
    @brief Request a commonly known name to use on the bus.
    
    All connections automatically register with a unique name on the D-BUS, but you
    can also request a commonly known name, normally in the form of a reverse
    domain name style name, e.g. "org.freedesktop.DBus". If some object already has this
    name, it will not be moved.
    
    @param name The commonly known name to request
    @returns false if the name was not successfully acquired.
    @todo FIXME: Does not support RequestName flags
    */
    bool requestName( const QString & name );
    
  signals:
    /**
    Emitted when a reply is received from a message sent using sendWithReply().
    
    @param message The original message
    @param reply The reply
    */
    void replyReceived( const DBusQt::Message& message, const DBusQt::Message& reply );
    
  public slots:
    void open( const QString& );
    void close();
    void flush();
    void send( const Message& );
    /**
    Sends a method call asynchronously.
    
    The message will be sent over this connection. If and when a reply is
    received, replyReceived() will be emitted with the reply message.
    
    @param message The message to send
    */
    void sendWithReply( const Message& message );
    Message sendWithReplyAndBlock( const Message& );

  protected slots:
    void dispatchRead();

  protected:
    void init( const QString& host );
    virtual void *virtual_hook( int id, void *data );

  private:
    friend class Internal::Integrator;
    DBusConnection *connection() const;
    Connection( DBusConnection *connection, QObject *parent );
    
    void emitReplyReceived( const Message &message, const Message &reply );
    static void pendingCallNotify( DBusPendingCall *pending, void/*SentMessage*/ *data );
    static void deleteSentMessage( void/*SentMessage*/ *message );
    
    static DBusHandlerResult messageHandler( DBusConnection *connection, DBusMessage *message, void *user_data );
    static void unregisterHandler( DBusConnection *connection, void *user_data );
  private:
    class Private;
    Private *d;


  };

}


#endif
