/* -*- mode: C++; c-file-style: "gnu" -*- */
/* message.h: Qt wrapper for DBusMessage
 *
 * Copyright (C) 2003  Zack Rusin <zack@kde.org>
 *               2005  James Thorniley <james@uncommonlygood.co.uk>
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef DBUS_QT_MESSAGE_H
#define DBUS_QT_MESSAGE_H

#include "dbus/dbus.h"

#ifdef __cplusplus //prevents Doxygen from deciding that Message is a template class
template <class T> class QValueList;
#endif
class QString;

namespace DBusQt {

  class Argument;

  /**
  A message that can be sent over the D-BUS protocol.
  
  <p>Messages must be one of the four types described by {@link #MessageType}.
  Once a message has been created, it should be sent using a Connection object.
  
  */
  class Message
  {
  public:

    /**
    Types a D-BUS message can take.
    */
    enum MessageType {
      Invalid = DBUS_MESSAGE_TYPE_INVALID,
      ///A message to invoke a method on a remote object
      MethodCall = DBUS_MESSAGE_TYPE_METHOD_CALL, 
      ///A message containing a reply from a remote method invocation
      MethodReturn = DBUS_MESSAGE_TYPE_METHOD_RETURN, 
      ///A message containing an error message
      Error = DBUS_MESSAGE_TYPE_ERROR,  
       ///A message containing an emitted signal
      Signal = DBUS_MESSAGE_TYPE_SIGNAL 
    };
    
    
    /**
    Construct a generic message of the given type.
 
    @param type The MessageType for the new message
    */
    Message( MessageType type );
    Message( const QString& service, const QString& path,
             const QString& interface, const QString& method );
    Message( const Message& replayingTo );
    Message( const QString& path, const QString& interface,
             const QString& name );
    Message( const Message& replayingTo, const QString& errorName,
             const QString& errorMessage );
    ~Message();
    
    MessageType type() const;

    void setPath( const QString& );
    QString path() const;
    
    void setInterface( const QString& );
    QString interface() const;

    void setMember( const QString& );
    QString member() const;

    void setErrorName( const QString& );
    QString errorName() const;

    void setDestination( const QString& );
    QString destination() const;

    bool    setSender( const QString& sender );
    QString    sender() const;

    QString signature() const;

    QValueList<Argument> *arguments() const;
    
    bool operator==( const Message &msg ) const;
    
  protected:
    friend class Connection; 
    DBusMessage* message() const;
    Message( DBusMessage * ); //hide this one from the public implementation

  private:
    class Private;
    Private *d;
  };

}

#endif
